@@ -27,38 +27,13 @@
 #include <boost/algorithm/string.hpp>
 
 #include <snapper/AppUtil.h>
+#include <snapper/SystemCmd.h>
 
 #include "utils/text.h"
 
 #include "misc.h"
 
 
-Snapshots::iterator
-read_num(Snapper* snapper, const string& str)
-{
-    Snapshots& snapshots = snapper->getSnapshots();
-
-    istringstream s(str);
-    unsigned int num = 0;
-    s >> num;
-
-    if (s.fail() || !s.eof())
-    {
-	cerr << sformat(_("Invalid snapshot '%s'."), str.c_str()) << endl;
-	exit(EXIT_FAILURE);
-    }
-
-    Snapshots::iterator snap = snapshots.find(num);
-    if (snap == snapshots.end())
-    {
-	cerr << sformat(_("Snapshot '%u' not found."), num) << endl;
-	exit(EXIT_FAILURE);
-    }
-
-    return snap;
-}
-
-
 unsigned int
 read_num(const string& str)
 {
@@ -76,29 +51,6 @@ read_num(const string& str)
 }
 
 
-pair<unsigned int, unsigned int>
-read_nums(const string& str, const string& delim)
-{
-    string::size_type pos = str.find(delim);
-    if (pos == string::npos)
-    {
-	cerr << _("Invalid snapshots.") << endl;
-	exit(EXIT_FAILURE);
-    }
-
-    unsigned int num1 = read_num(str.substr(0, pos));
-    unsigned int num2 = read_num(str.substr(pos + delim.size()));
-
-    if (num1 == num2)
-    {
-	cerr << _("Identical snapshots.") << endl;
-	exit(EXIT_FAILURE);
-    }
-
-    return pair<unsigned int, unsigned int>(num1, num2);
-}
-
-
 map<string, string>
 read_userdata(const string& s, const map<string, string>& old)
 {
@@ -163,9 +115,9 @@ show_userdata(const map<string, string>& userdata)
 
 
 map<string, string>
-read_configdata(const list<string>& l, const map<string, string>& old)
+read_configdata(const vector<string>& v, const map<string, string>& old)
 {
-    if (l.empty())
+    if (v.empty())
     {
 	cerr << _("Empty configdata.") << endl;
 	exit(EXIT_FAILURE);
@@ -173,7 +125,7 @@ read_configdata(const list<string>& l, const map<string, string>& old)
 
     map<string, string> configdata = old;
 
-    for (list<string>::const_iterator it = l.begin(); it != l.end(); ++it)
+    for (vector<string>::const_iterator it = v.begin(); it != v.end(); ++it)
     {
 	string::size_type pos = it->find("=");
 	if (pos == string::npos)
@@ -209,3 +161,27 @@ username(uid_t uid)
 
     return username;
 }
+
+
+Differ::Differ()
+    : command(DIFFBIN " --new-file --unified"), extensions()
+{
+}
+
+
+void
+Differ::run(const string& f1, const string& f2) const
+{
+    string tmp = command;
+    if (!extensions.empty())
+	tmp += " " + extensions;
+    tmp += " " + quote(f1) + " " + quote(f2);
+
+    SystemCmd cmd(tmp);
+
+    for (const string& line : cmd.stdout())
+	cout << line << endl;
+
+    for (const string& line : cmd.stderr())
+	cerr << line << endl;
+}
