@@ -1,6 +1,6 @@
 /*
  * Copyright (c) [2004-2015] Novell, Inc.
- * Copyright (c) 2016 SUSE LLC
+ * Copyright (c) [2016-2020] SUSE LLC
  *
  * All Rights Reserved.
  *
@@ -223,14 +223,14 @@ namespace snapper
     string
     stringerror(int errnum)
     {
-#if (_POSIX_C_SOURCE >= 200112L || _XOPEN_SOURCE >= 600) && ! _GNU_SOURCE
+#if (_POSIX_C_SOURCE >= 200112L) && ! _GNU_SOURCE
 	char buf1[100];
-	if (strerror_r(errno, buf1, sizeof(buf1)-1) == 0)
+	if (strerror_r(errnum, buf1, sizeof(buf1) - 1) == 0)
 	    return string(buf1);
 	return string("strerror failed");
 #else
 	char buf1[100];
-	const char* buf2 = strerror_r(errno, buf1, sizeof(buf1)-1);
+	const char* buf2 = strerror_r(errnum, buf1, sizeof(buf1) - 1);
 	return string(buf2);
 #endif
     }
@@ -415,4 +415,11 @@ namespace snapper
 	return s << fixed << sw.read() << "s";
     }
 
+
+    bool
+    Uuid::operator==(const Uuid& rhs) const
+    {
+	return std::equal(std::begin(value), std::end(value), std::begin(rhs.value));
+    }
+
 }
