@@ -130,6 +130,7 @@ namespace snapper
 		case EEXIST:
 		    SN_THROW(CreateConfigFailedException("creating btrfs subvolume .snapshots failed "
 							 "since it already exists"));
+                    break;
 
 		default:
 		    SN_THROW(CreateConfigFailedException("creating btrfs subvolume .snapshots failed"));
@@ -1235,11 +1236,15 @@ namespace snapper
     bool
     StreamProcessor::dumper(int fd)
     {
+	FdCloser fd_closer(fd);
+
+	unsigned int iterations = 0;
+
 	while (true)
 	{
 	    boost::this_thread::interruption_point();
 
-	     // remove the fourth parameter for older versions of libbtrfs
+	    // remove the fourth parameter for older versions of libbtrfs
 	    int r;
 
 #if BTRFS_LIB_VERSION < 101
@@ -1248,9 +1253,15 @@ namespace snapper
 	    r = btrfs_read_and_process_send_stream(fd, &send_ops, &*this, 0, 1);
 #endif
 
-	    if (r < 0)
+	    // Only return an error when r == -ENODATA if this was the
+	    // first call to btrfs_read_and_process_send_stream(). See
+	    // https://github.com/openSUSE/snapper/pull/438,
+	    // https://bugzilla.suse.com/show_bug.cgi?id=1111414 and
+	    // the btrfs-progs source code.
+
+	    if (r < 0 && !(r == -ENODATA && iterations > 0))
 	    {
-		y2err("btrfs_read_and_process_send_stream failed");
+		y2err("btrfs_read_and_process_send_stream failed " << r);
 
 #if BOOST_VERSION < 104100
 		dumper_ret = false;
@@ -1268,6 +1279,7 @@ namespace snapper
 		return true;
 	    }
 
+	    ++iterations;
 	}
     }
 
@@ -1283,6 +1295,10 @@ namespace snapper
 	    SN_THROW(BtrfsSendReceiveException());
 	}
 
+	// Use RAII to help close fds.
+	FdCloser fd0_closer(pipefd[0]);
+	FdCloser fd1_closer(pipefd[1]);
+
 	struct btrfs_ioctl_send_args io_send;
 	memset(&io_send, 0, sizeof(io_send));
 	io_send.send_fd = pipefd[1];
@@ -1298,18 +1314,18 @@ namespace snapper
 
 	boost::thread task(boost::move(pt));
 
+	fd0_closer.reset();
+
 	int r2 = ioctl(dir2.fd(), BTRFS_IOC_SEND, &io_send);
 	if (r2 < 0)
 	{
 	    y2err("send ioctl failed errno:" << errno << " (" << stringerror(errno) << ")");
 	}
 
-	close(pipefd[1]);
+	fd1_closer.close();
 
 	uf.wait();
 
-	close(pipefd[0]);
-
 	if (r2 < 0 || !uf.get())
 	{
 	    SN_THROW(BtrfsSendReceiveException());
@@ -1319,18 +1335,18 @@ namespace snapper
 
 	boost::thread dumper_thread(boost::bind(&StreamProcessor::dumper, this, pipefd[0]));
 
+	fd0_closer.reset();
+
 	int r2 = ioctl(dir2.fd(), BTRFS_IOC_SEND, &io_send);
 	if (r2 < 0)
 	{
 	    y2err("send ioctl failed errno:" << errno << " (" << stringerror(errno) << ")");
 	}
 
-	close(pipefd[1]);
+	fd1_closer.close();
 
 	dumper_thread.join();
 
-	close(pipefd[0]);
-
 	if (r2 < 0 || !dumper_ret)
 	{
 	    SN_THROW(BtrfsSendReceiveException());
@@ -1463,6 +1479,9 @@ namespace snapper
 
 	unsigned int num = stoi(rx.cap(1));
 
+	if (!checkSnapshot(num))
+	    return make_pair(false, 0);
+
 	if (get_id(openSnapshotDir(num).fd()) != id)
 	    return make_pair(false, 0);
 
@@ -1544,40 +1563,35 @@ namespace snapper
     bool
     Btrfs::isDefault(unsigned int num) const
     {
-	SN_THROW(UnsupportedException());
-	__builtin_unreachable();
+	return Filesystem::isDefault(num);
     }
 
 
     std::pair<bool, unsigned int>
     Btrfs::getDefault() const
     {
-	SN_THROW(UnsupportedException());
-	__builtin_unreachable();
+	return Filesystem::getDefault();
     }
 
 
     void
     Btrfs::setDefault(unsigned int num) const
     {
-	SN_THROW(UnsupportedException());
-	__builtin_unreachable();
+	Filesystem::setDefault(num);
     }
 
 
     std::pair<bool, unsigned int>
     Btrfs::getActive() const
     {
-	SN_THROW(UnsupportedException());
-	__builtin_unreachable();
+	return Filesystem::getActive();
     }
 
 
     bool
     Btrfs::isActive(unsigned int num) const
     {
-	SN_THROW(UnsupportedException());
-	__builtin_unreachable();
+	return Filesystem::isActive(num);
     }
 
 #endif
