@@ -1,5 +1,6 @@
 /*
- * Copyright (c) [2011-2014] Novell, Inc.
+ * Copyright (c) [2011-2015] Novell, Inc.
+ * Copyright (c) [2016-2020] SUSE LLC
  *
  * All Rights Reserved.
  *
@@ -36,12 +37,13 @@
 #include "snapper/AppUtil.h"
 #include "snapper/XmlFile.h"
 #include "snapper/Filesystem.h"
+#include "snapper/Btrfs.h"
 #include "snapper/Enum.h"
 #include "snapper/SnapperTmpl.h"
 #include "snapper/SnapperDefines.h"
 #include "snapper/Exception.h"
-#include "snapper/SystemCmd.h"
 #include "snapper/Regex.h"
+#include "snapper/Hooks.h"
 
 
 namespace snapper
@@ -58,7 +60,8 @@ namespace snapper
 
 	s << " date:\"" << datetime(snapshot.date, true, true) << "\"";
 
-	s << "uid:" << snapshot.uid;
+	if (snapshot.uid != 0)
+	    s << "uid:" << snapshot.uid;
 
 	if (!snapshot.description.empty())
 	    s << " description:\"" << snapshot.description << "\"";
@@ -75,8 +78,7 @@ namespace snapper
 
     Snapshot::Snapshot(const Snapper* snapper, SnapshotType type, unsigned int num, time_t date)
 	: snapper(snapper), type(type), num(num), date(date), uid(0), pre_num(0),
-	  info_modified(false), mount_checked(false), mount_user_request(false),
-	  mount_use_count(0)
+	  mount_checked(false), mount_user_request(false), mount_use_count(0)
     {
     }
 
@@ -105,7 +107,7 @@ namespace snapper
     Snapshot::openInfoDir() const
     {
 	if (isCurrent())
-	    throw IllegalSnapshotException();
+	    SN_THROW(IllegalSnapshotException());
 
 	SDir infos_dir = snapper->openInfosDir();
 	return SDir(infos_dir, decString(num));
@@ -132,56 +134,51 @@ namespace snapper
     }
 
 
-    void
-    Snapshot::setUid(uid_t val)
+    bool
+    Snapshot::isDefault() const
     {
-	if (isCurrent())
-	    throw IllegalSnapshotException();
-
-	uid = val;
-	info_modified = true;
+	return snapper->getFilesystem()->isDefault(num);
     }
 
 
     void
-    Snapshot::setDescription(const string& val)
+    Snapshot::setDefault() const
     {
-	if (isCurrent())
-	    throw IllegalSnapshotException();
-
-	description = val;
-	info_modified = true;
+	return snapper->getFilesystem()->setDefault(num);
     }
 
 
-    void
-    Snapshot::setCleanup(const string& val)
+    bool
+    Snapshot::isActive() const
     {
-	if (isCurrent())
-	    throw IllegalSnapshotException();
-
-	cleanup = val;
-	info_modified = true;
+	return !isCurrent() && snapper->getFilesystem()->isActive(num);
     }
 
 
-    void
-    Snapshot::setUserdata(const map<string, string>& val)
+    uint64_t
+    Snapshot::getUsedSpace() const
     {
-	if (isCurrent())
-	    throw IllegalSnapshotException();
+#ifdef ENABLE_BTRFS_QUOTA
 
-	for (map<string, string>::const_iterator it = val.begin(); it != val.end(); ++it)
-	{
-	    if (it->first.empty() || it->first.find_first_of(",=") != string::npos)
-		throw InvalidUserdataException();
+	const Btrfs* btrfs = dynamic_cast<const Btrfs*>(snapper->getFilesystem());
+	if (!btrfs)
+	    SN_THROW(QuotaException("quota only supported with btrfs"));
 
-	    if (it->second.find_first_of(",=") != string::npos)
-		throw InvalidUserdataException();
-	}
+	SDir general_dir = btrfs->openGeneralDir();
 
-	userdata = val;
-	info_modified = true;
+	subvolid_t subvolid = get_id(openSnapshotDir().fd());
+	qgroup_t qgroup = calc_qgroup(0, subvolid);
+
+	QGroupUsage qgroup_usage = qgroup_query_usage(general_dir.fd(), qgroup);
+
+	return qgroup_usage.exclusive;
+
+#else
+
+	SN_THROW(QuotaException("not implemented"));
+	__builtin_unreachable();
+
+#endif
     }
 
 
@@ -204,8 +201,7 @@ namespace snapper
 		int fd = info_dir.open("info.xml", O_NOFOLLOW | O_CLOEXEC);
 		XmlFile file(fd, "");
 
-		const xmlNode* root = file.getRootElement();
-		const xmlNode* node = getChildNode(root, "snapshot");
+		const xmlNode* node = file.getRootElement();
 
 		string tmp;
 
@@ -340,10 +336,10 @@ namespace snapper
 	for (map<string, string>::const_iterator it = userdata.begin(); it != userdata.end(); ++it)
 	{
 	    if (it->first.empty() || it->first.find_first_of(",=") != string::npos)
-		throw InvalidUserdataException();
+		SN_THROW(InvalidUserdataException());
 
 	    if (it->second.find_first_of(",=") != string::npos)
-		throw InvalidUserdataException();
+		SN_THROW(InvalidUserdataException());
 	}
     }
 
@@ -374,7 +370,7 @@ namespace snapper
     Snapshots::findPost(const_iterator pre)
     {
 	if (pre == entries.end() || pre->isCurrent() || pre->getType() != PRE)
-	    throw IllegalSnapshotException();
+	    SN_THROW(IllegalSnapshotException());
 
 	for (iterator it = begin(); it != end(); ++it)
 	{
@@ -390,7 +386,7 @@ namespace snapper
     Snapshots::findPost(const_iterator pre) const
     {
 	if (pre == entries.end() || pre->isCurrent() || pre->getType() != PRE)
-	    throw IllegalSnapshotException();
+	    SN_THROW(IllegalSnapshotException());
 
 	for (const_iterator it = begin(); it != end(); ++it)
 	{
@@ -406,7 +402,7 @@ namespace snapper
     Snapshots::findPre(const_iterator post)
     {
 	if (post == entries.end() || post->isCurrent() || post->getType() != POST)
-	    throw IllegalSnapshotException();
+	    SN_THROW(IllegalSnapshotException());
 
 	return find(post->pre_num);
     }
@@ -416,7 +412,7 @@ namespace snapper
     Snapshots::findPre(const_iterator post) const
     {
 	if (post == entries.end() || post->isCurrent() || post->getType() != POST)
-	    throw IllegalSnapshotException();
+	    SN_THROW(IllegalSnapshotException());
 
 	return find(post->pre_num);
     }
@@ -442,8 +438,8 @@ namespace snapper
 	    if (errno == EEXIST)
 		continue;
 
-	    y2err("mkdir failed errno:" << errno << " (" << stringerror(errno) << ")");
-	    throw IOErrorException();
+	    SN_THROW(IOErrorException(sformat("mkdir failed errno:%d (%s)", errno,
+					      stringerror(errno).c_str())));
 	}
 
 	infos_dir.chmod(decString(num), 0755, 0);
@@ -452,17 +448,6 @@ namespace snapper
     }
 
 
-    void
-    Snapshot::flushInfo()
-    {
-	if (!info_modified)
-	    return;
-
-	writeInfo();
-	info_modified = false;
-    }
-
-
     void
     Snapshot::writeInfo() const
     {
@@ -476,7 +461,8 @@ namespace snapper
 
 	setChildValue(node, "date", datetime(date, true, true));
 
-	setChildValue(node, "uid", uid);
+	if (uid != 0)
+	    setChildValue(node, "uid", uid);
 
 	if (type == POST)
 	    setChildValue(node, "pre_num", pre_num);
@@ -502,11 +488,11 @@ namespace snapper
 	xml.save(info_dir.mktemp(tmp_name));
 
 	if (info_dir.rename(tmp_name, file_name) != 0)
-	{
-	    y2err("rename info.xml failed infoDir: " << info_dir.fullname() << " errno: " <<
-		  errno << " (" << stringerror(errno) << ")");
-	    throw IOErrorException();
-	}
+	    SN_THROW(IOErrorException(sformat("rename info.xml failed infoDir:%s errno:%d (%s)",
+					      info_dir.fullname().c_str(), errno,
+					      stringerror(errno).c_str())));
+
+	info_dir.fsync();
     }
 
 
@@ -514,7 +500,7 @@ namespace snapper
     Snapshot::mountFilesystemSnapshot(bool user_request) const
     {
 	if (isCurrent())
-	    throw IllegalSnapshotException();
+	    SN_THROW(IllegalSnapshotException());
 
 	if (!mount_checked)
 	{
@@ -535,7 +521,7 @@ namespace snapper
     Snapshot::umountFilesystemSnapshot(bool user_request) const
     {
 	if (isCurrent())
-	    throw IllegalSnapshotException();
+	    SN_THROW(IllegalSnapshotException());
 
 	if (!mount_checked)
 	{
@@ -565,12 +551,13 @@ namespace snapper
 
 
     void
-    Snapshot::createFilesystemSnapshot(unsigned int num_parent, bool read_only) const
+    Snapshot::createFilesystemSnapshot(unsigned int num_parent, bool read_only, bool empty) const
     {
 	if (isCurrent())
-	    throw IllegalSnapshotException();
+	    SN_THROW(IllegalSnapshotException());
 
-	snapper->getFilesystem()->createSnapshot(num, num_parent, read_only);
+	snapper->getFilesystem()->createSnapshot(num, num_parent, read_only, !cleanup.empty(),
+						 empty);
     }
 
 
@@ -578,133 +565,103 @@ namespace snapper
     Snapshot::createFilesystemSnapshotOfDefault(bool read_only) const
     {
 	if (isCurrent())
-	    throw IllegalSnapshotException();
+	    SN_THROW(IllegalSnapshotException());
 
-	snapper->getFilesystem()->createSnapshotOfDefault(num, read_only);
+	snapper->getFilesystem()->createSnapshotOfDefault(num, read_only, !cleanup.empty());
     }
 
 
     void
     Snapshot::deleteFilesystemSnapshot() const
     {
-	if (isCurrent())
-	    throw IllegalSnapshotException();
-
 	snapper->getFilesystem()->umountSnapshot(num);
 	snapper->getFilesystem()->deleteSnapshot(num);
     }
 
 
     Snapshots::iterator
-    Snapshots::createSingleSnapshot(string description)
+    Snapshots::createSingleSnapshot(const SCD& scd)
     {
-	return createSingleSnapshot(0, description, "", map<string, string>());
-    }
-
-
-    Snapshots::iterator
-    Snapshots::createPreSnapshot(string description)
-    {
-	return createPreSnapshot(0, description, "", map<string, string>());
-    }
-
-
-    Snapshots::iterator
-    Snapshots::createPostSnapshot(string description, Snapshots::const_iterator pre)
-    {
-	return createPostSnapshot(pre, 0, description, "", map<string, string>());
-    }
-
-
-    Snapshots::iterator
-    Snapshots::createSingleSnapshot(uid_t uid, const string& description, const string& cleanup,
-				    const map<string, string>& userdata)
-    {
-	checkUserdata(userdata);
+	checkUserdata(scd.userdata);
 
 	Snapshot snapshot(snapper, SINGLE, nextNumber(), time(NULL));
-	snapshot.uid = uid;
-	snapshot.description = description;
-	snapshot.cleanup = cleanup;
-	snapshot.userdata = userdata;
+	snapshot.uid = scd.uid;
+	snapshot.description = scd.description;
+	snapshot.cleanup = scd.cleanup;
+	snapshot.userdata = scd.userdata;
 
-	return createHelper(snapshot, getSnapshotCurrent(), true);
+	return createHelper(snapshot, getSnapshotCurrent(), scd.read_only, scd.empty);
     }
 
 
     Snapshots::iterator
-    Snapshots::createSingleSnapshot(const_iterator parent, bool read_only, uid_t uid,
-				    const string& description, const string& cleanup,
-				    const map<string, string>& userdata)
+    Snapshots::createSingleSnapshot(const_iterator parent, const SCD& scd)
     {
-	checkUserdata(userdata);
+	checkUserdata(scd.userdata);
 
 	Snapshot snapshot(snapper, SINGLE, nextNumber(), time(NULL));
-	snapshot.uid = uid;
-	snapshot.description = description;
-	snapshot.cleanup = cleanup;
-	snapshot.userdata = userdata;
+	snapshot.uid = scd.uid;
+	snapshot.description = scd.description;
+	snapshot.cleanup = scd.cleanup;
+	snapshot.userdata = scd.userdata;
 
-	return createHelper(snapshot, parent, read_only);
+	return createHelper(snapshot, parent, scd.read_only);
     }
 
 
     Snapshots::iterator
-    Snapshots::createSingleSnapshotOfDefault(bool read_only, uid_t uid,
-					     const string& description, const string& cleanup,
-					     const map<string, string>& userdata)
+    Snapshots::createSingleSnapshotOfDefault(const SCD& scd)
     {
-	checkUserdata(userdata);
+	checkUserdata(scd.userdata);
 
 	Snapshot snapshot(snapper, SINGLE, nextNumber(), time(NULL));
-	snapshot.uid = uid;
-	snapshot.description = description;
-	snapshot.cleanup = cleanup;
-	snapshot.userdata = userdata;
+	snapshot.uid = scd.uid;
+	snapshot.description = scd.description;
+	snapshot.cleanup = scd.cleanup;
+	snapshot.userdata = scd.userdata;
 
-	return createHelper(snapshot, end(), read_only);
+	return createHelper(snapshot, end(), scd.read_only);
     }
 
 
     Snapshots::iterator
-    Snapshots::createPreSnapshot(uid_t uid, const string& description, const string& cleanup,
-				 const map<string, string>& userdata)
+    Snapshots::createPreSnapshot(const SCD& scd)
     {
-	checkUserdata(userdata);
+	checkUserdata(scd.userdata);
 
 	Snapshot snapshot(snapper, PRE, nextNumber(), time(NULL));
-	snapshot.uid = uid;
-	snapshot.description = description;
-	snapshot.cleanup = cleanup;
-	snapshot.userdata = userdata;
+	snapshot.uid = scd.uid;
+	snapshot.description = scd.description;
+	snapshot.cleanup = scd.cleanup;
+	snapshot.userdata = scd.userdata;
 
-	return createHelper(snapshot, getSnapshotCurrent(), true);
+	return createHelper(snapshot, getSnapshotCurrent(), scd.read_only);
     }
 
 
     Snapshots::iterator
-    Snapshots::createPostSnapshot(Snapshots::const_iterator pre, uid_t uid, const string& description,
-				  const string& cleanup, const map<string, string>& userdata)
+    Snapshots::createPostSnapshot(Snapshots::const_iterator pre, const SCD& scd)
     {
 	if (pre == entries.end() || pre->isCurrent() || pre->getType() != PRE ||
 	    findPost(pre) != entries.end())
-	    throw IllegalSnapshotException();
+	    SN_THROW(IllegalSnapshotException());
 
-	checkUserdata(userdata);
+	checkUserdata(scd.userdata);
 
 	Snapshot snapshot(snapper, POST, nextNumber(), time(NULL));
 	snapshot.pre_num = pre->getNum();
-	snapshot.uid = uid;
-	snapshot.description = description;
-	snapshot.cleanup = cleanup;
-	snapshot.userdata = userdata;
+	snapshot.uid = scd.uid;
+	snapshot.description = scd.description;
+	snapshot.cleanup = scd.cleanup;
+	snapshot.userdata = scd.userdata;
 
-	return createHelper(snapshot, getSnapshotCurrent(), true);
+	return createHelper(snapshot, getSnapshotCurrent(), scd.read_only);
     }
 
 
     Snapshots::iterator
-    Snapshots::createHelper(Snapshot& snapshot, const_iterator parent, bool read_only)
+    Snapshots::createHelper(Snapshot& snapshot, const_iterator parent, bool read_only,
+			    bool empty)
     {
 	// parent == end indicates the btrfs default subvolume. Unclean, but
 	// adding a special snapshot like current needs too many API changes.
@@ -712,15 +669,18 @@ namespace snapper
 	try
 	{
 	    if (parent != end())
-		snapshot.createFilesystemSnapshot(parent->getNum(), read_only);
+		snapshot.createFilesystemSnapshot(parent->getNum(), read_only, empty);
 	    else
 		snapshot.createFilesystemSnapshotOfDefault(read_only);
 	}
 	catch (const CreateSnapshotFailedException& e)
 	{
+	    SN_CAUGHT(e);
+
 	    SDir infos_dir = snapper->openInfosDir();
 	    infos_dir.unlink(decString(snapshot.getNum()), AT_REMOVEDIR);
-	    throw;
+
+	    SN_RETHROW(e);
 	}
 
 	try
@@ -729,19 +689,16 @@ namespace snapper
 	}
 	catch (const IOErrorException& e)
 	{
+	    SN_CAUGHT(e);
+
 	    snapshot.deleteFilesystemSnapshot();
 	    SDir infos_dir = snapper->openInfosDir();
 	    infos_dir.unlink(decString(snapshot.getNum()), AT_REMOVEDIR);
-	    throw;
+
+	    SN_RETHROW(e);
 	}
 
-#ifdef ENABLE_ROLLBACK
-	if (snapper->subvolumeDir() == "/" && snapper->getFilesystem()->fstype() == "btrfs" &&
-	    access("/usr/lib/snapper/plugins/grub", X_OK) == 0)
-	{
-	    SystemCmd cmd("/usr/lib/snapper/plugins/grub --refresh");
-	}
-#endif
+	Hooks::create_snapshot(snapper->subvolumeDir(), snapper->getFilesystem());
 
 	return entries.insert(entries.end(), snapshot);
     }
@@ -755,35 +712,29 @@ namespace snapper
 
 
     void
-    Snapshots::modifySnapshot(iterator snapshot, const string& description, const string& cleanup,
-			      const map<string, string>& userdata)
+    Snapshots::modifySnapshot(iterator snapshot, const SMD& smd)
     {
 	if (snapshot == entries.end() || snapshot->isCurrent())
-	    throw IllegalSnapshotException();
+	    SN_THROW(IllegalSnapshotException());
 
-	checkUserdata(userdata);
+	checkUserdata(smd.userdata);
 
-	snapshot->description = description;
-	snapshot->cleanup = cleanup;
-	snapshot->userdata = userdata;
+	snapshot->description = smd.description;
+	snapshot->cleanup = smd.cleanup;
+	snapshot->userdata = smd.userdata;
 
 	snapshot->writeInfo();
 
-#ifdef ENABLE_ROLLBACK
-	if (snapper->subvolumeDir() == "/" && snapper->getFilesystem()->fstype() == "btrfs" &&
-	    access("/usr/lib/snapper/plugins/grub", X_OK) == 0)
-	{
-	    SystemCmd cmd("/usr/lib/snapper/plugins/grub --refresh");
-	}
-#endif
+	Hooks::modify_snapshot(snapper->subvolumeDir(), snapper->getFilesystem());
     }
 
 
     void
     Snapshots::deleteSnapshot(iterator snapshot)
     {
-	if (snapshot == entries.end() || snapshot->isCurrent())
-	    throw IllegalSnapshotException();
+	if (snapshot == entries.end() || snapshot->isCurrent() || snapshot->isDefault() ||
+	    snapshot->isActive())
+	    SN_THROW(IllegalSnapshotException());
 
 	snapshot->deleteFilesystemSnapshot();
 
@@ -811,13 +762,34 @@ namespace snapper
 
 	entries.erase(snapshot);
 
-#ifdef ENABLE_ROLLBACK
-	if (snapper->subvolumeDir() == "/" && snapper->getFilesystem()->fstype() == "btrfs" &&
-	    access("/usr/lib/snapper/plugins/grub", X_OK) == 0)
-	{
-	    SystemCmd cmd("/usr/lib/snapper/plugins/grub --refresh");
-	}
-#endif
+	Hooks::delete_snapshot(snapper->subvolumeDir(), snapper->getFilesystem());
+    }
+
+
+    Snapshots::iterator
+    Snapshots::getDefault()
+    {
+	std::pair<bool, unsigned int> tmp = snapper->getFilesystem()->getDefault();
+
+	return tmp.first ? find(tmp.second) : end();
+    }
+
+
+    Snapshots::const_iterator
+    Snapshots::getDefault() const
+    {
+	std::pair<bool, unsigned int> tmp = snapper->getFilesystem()->getDefault();
+
+	return tmp.first ? find(tmp.second) : end();
+    }
+
+
+    Snapshots::const_iterator
+    Snapshots::getActive() const
+    {
+	std::pair<bool, unsigned int> tmp = snapper->getFilesystem()->getActive();
+
+	return tmp.first ? find(tmp.second) : end();
     }
 
 
